import os

from quixote import get_publisher

from qommon.storage import StorableObject
from qommon import get_cfg

class User(StorableObject):
    _names = 'users'

    name = None
    email = None
    roles = None
    is_admin = False
    anonymous = False
    form_data = None  # dumping ground for custom fields

    name_identifiers = None
    identification_token = None
    lasso_dump = None

    last_seen = None

    def __init__(self, name = None):
        StorableObject.__init__(self)
        self.name = name
        self.name_identifiers = []
        self.roles = []

    def migrate(self):
        changed = False

        if self.roles and 'site-admin' in self.roles:
            self.is_admin = True
            self.roles = [x for x in self.roles if x != 'site-admin']
            changed = True

        if changed:
            self.store()

    def get_formdef(cls):
        from admin.settings import UserFieldsFormDef
        return UserFieldsFormDef()
    get_formdef = classmethod(get_formdef)

    def get_display_name(self):
        if self.name:
            return self.name
        if self.email:
            return self.email
        return _('Unknown User')
    display_name = property(get_display_name)

    def set_attributes_from_formdata(self, formdata):
        users_cfg = get_cfg('users', {})

        if formdata.get('email'):
            self.email = formdata.get('email')

        field_email = users_cfg.get('field_email')
        if field_email:
            self.email = formdata.get(field_email)

        field_name_values = users_cfg.get('field_name')
        if type(field_name_values) is str: # it was a string in previous versions
            field_name_values = [field_name_values]

        if field_name_values:
            self.name = ' '.join([formdata.get(x) for x in field_name_values if formdata.get(x)])

    def can_go_in_admin(self):
        return self.is_admin

    def can_go_in_backoffice(self):
        if self.is_admin:
            return True
        from formdef import FormDef
        formdefs = FormDef.select(lambda x: not x.disabled)
        for formdef_id in FormDef.keys():
            formdef = FormDef.get(formdef_id, ignore_errors=True)
            if formdef and not formdef.disabled and formdef.receiver_id in (self.roles or []):
                return True
        return False

